# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .schema_language import SchemaLanguage
from .git_commit_ref_param import GitCommitRefParam

__all__ = ["ConfigSchemaCreateParams", "ConfigTypeRef", "Document", "GitCommit"]


class ConfigSchemaCreateParams(TypedDict, total=False):
    config_type_ref: Required[ConfigTypeRef]
    """A reference to a config type.

    At least one of `id` or `slug` must be provided. When both are provided, `id`
    takes precedence and `slug` is ignored.
    """

    documents: Required[Iterable[Document]]
    """The schema document files."""

    format: Required[Literal["json", "yaml", "cue"]]

    language: Required[SchemaLanguage]

    expand: List[Literal["documents", "config_type"]]
    """Fields to expand on the config schema resource."""

    git_commit: GitCommit
    """The git commit to link to this config schema."""

    instance_filepath: str
    """The file path for config instances created from this schema."""


class ConfigTypeRef(TypedDict, total=False):
    """A reference to a config type.

    At least one of `id` or `slug` must be provided. When both are provided, `id` takes precedence and `slug` is ignored.
    """

    id: str
    """ID of the config type. Takes precedence over `slug` when both are provided."""

    slug: str
    """An immutable, code-friendly name for the config type.

    Used only when `id` is not provided.
    """


class Document(TypedDict, total=False):
    data: Required[str]
    """The raw document content."""

    name: Required[str]
    """The name of this document."""


class GitCommit(TypedDict, total=False):
    """The git commit to link to this config schema."""

    commit_ref: Required[GitCommitRefParam]
    """A reference to a git commit.

    At least one of `id` or `sha` must be provided. When both are provided, `id`
    takes precedence and `sha` is ignored.
    """

    schema_filepaths: Required[SequenceNotStr[str]]
    """The file paths of the config schema files relative to the git repository root.

    JSON Schema only supports a single file for a schema. CUE support multiple files
    via [packages](https://cuelang.org/docs/references/spec/#packages).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)
from .releases import (
    ReleasesResource,
    AsyncReleasesResource,
    ReleasesResourceWithRawResponse,
    AsyncReleasesResourceWithRawResponse,
    ReleasesResourceWithStreamingResponse,
    AsyncReleasesResourceWithStreamingResponse,
)
from .principal import (
    PrincipalResource,
    AsyncPrincipalResource,
    PrincipalResourceWithRawResponse,
    AsyncPrincipalResourceWithRawResponse,
    PrincipalResourceWithStreamingResponse,
    AsyncPrincipalResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)
from .git_commits import (
    GitCommitsResource,
    AsyncGitCommitsResource,
    GitCommitsResourceWithRawResponse,
    AsyncGitCommitsResourceWithRawResponse,
    GitCommitsResourceWithStreamingResponse,
    AsyncGitCommitsResourceWithStreamingResponse,
)
from .config_types import (
    ConfigTypesResource,
    AsyncConfigTypesResource,
    ConfigTypesResourceWithRawResponse,
    AsyncConfigTypesResourceWithRawResponse,
    ConfigTypesResourceWithStreamingResponse,
    AsyncConfigTypesResourceWithStreamingResponse,
)
from .config_schemas import (
    ConfigSchemasResource,
    AsyncConfigSchemasResource,
    ConfigSchemasResourceWithRawResponse,
    AsyncConfigSchemasResourceWithRawResponse,
    ConfigSchemasResourceWithStreamingResponse,
    AsyncConfigSchemasResourceWithStreamingResponse,
)
from .config_instances import (
    ConfigInstancesResource,
    AsyncConfigInstancesResource,
    ConfigInstancesResourceWithRawResponse,
    AsyncConfigInstancesResourceWithRawResponse,
    ConfigInstancesResourceWithStreamingResponse,
    AsyncConfigInstancesResourceWithStreamingResponse,
)

__all__ = [
    "ConfigInstancesResource",
    "AsyncConfigInstancesResource",
    "ConfigInstancesResourceWithRawResponse",
    "AsyncConfigInstancesResourceWithRawResponse",
    "ConfigInstancesResourceWithStreamingResponse",
    "AsyncConfigInstancesResourceWithStreamingResponse",
    "ConfigSchemasResource",
    "AsyncConfigSchemasResource",
    "ConfigSchemasResourceWithRawResponse",
    "AsyncConfigSchemasResourceWithRawResponse",
    "ConfigSchemasResourceWithStreamingResponse",
    "AsyncConfigSchemasResourceWithStreamingResponse",
    "ConfigTypesResource",
    "AsyncConfigTypesResource",
    "ConfigTypesResourceWithRawResponse",
    "AsyncConfigTypesResourceWithRawResponse",
    "ConfigTypesResourceWithStreamingResponse",
    "AsyncConfigTypesResourceWithStreamingResponse",
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
    "DevicesResource",
    "AsyncDevicesResource",
    "DevicesResourceWithRawResponse",
    "AsyncDevicesResourceWithRawResponse",
    "DevicesResourceWithStreamingResponse",
    "AsyncDevicesResourceWithStreamingResponse",
    "GitCommitsResource",
    "AsyncGitCommitsResource",
    "GitCommitsResourceWithRawResponse",
    "AsyncGitCommitsResourceWithRawResponse",
    "GitCommitsResourceWithStreamingResponse",
    "AsyncGitCommitsResourceWithStreamingResponse",
    "PrincipalResource",
    "AsyncPrincipalResource",
    "PrincipalResourceWithRawResponse",
    "AsyncPrincipalResourceWithRawResponse",
    "PrincipalResourceWithStreamingResponse",
    "AsyncPrincipalResourceWithStreamingResponse",
    "ReleasesResource",
    "AsyncReleasesResource",
    "ReleasesResourceWithRawResponse",
    "AsyncReleasesResourceWithRawResponse",
    "ReleasesResourceWithStreamingResponse",
    "AsyncReleasesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .instance_content_param import InstanceContentParam

__all__ = ["ConfigInstanceCreateParams"]


class ConfigInstanceCreateParams(TypedDict, total=False):
    config_schema_id: Required[str]
    """The ID of the config schema which this config instance must adhere to."""

    content: Required[InstanceContentParam]

    expand: List[Literal["content", "config_schema", "config_type"]]
    """Fields to expand on the config instance resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .config_type import ConfigType
from .shared.paginated_list import PaginatedList

__all__ = ["ConfigTypeListResponse"]


class ConfigTypeListResponse(PaginatedList):
    data: List[ConfigType]
    """The list of config types."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .device import Device
from .release import Release
from .._models import BaseModel
from .config_instance import ConfigInstance

__all__ = ["Deployment"]


class Deployment(BaseModel):
    id: str
    """ID of the deployment."""

    activity_status: Literal["drifted", "staged", "queued", "deployed", "archived"]
    """Last known activity state of the deployment.

    - Drifted: device's configurations have drifted since this deployment was
      staged, and the deployment needs to be reviewed before it can be deployed
    - Staged: is ready to be deployed
    - Queued: the deployment's config instances are waiting to be received by the
      device; will be deployed as soon as the device is online
    - Deployed: the deployment's config instances are currently available for
      consumption on the device
    - Archived: the deployment is available for historical reference but cannot be
      deployed and is not active on the device
    """

    created_at: datetime
    """Timestamp of when the device release was created."""

    description: str
    """The description of the deployment."""

    device_id: str
    """ID of the device."""

    error_status: Literal["none", "failed", "retrying"]
    """Last known error state of the deployment.

    - None: no errors
    - Retrying: an error has been encountered and the agent is retrying to reach the
      target status
    - Failed: a fatal error has been encountered; the deployment is archived and (if
      deployed) removed from the device
    """

    object: Literal["deployment"]
    """The object type, which is always `deployment`."""

    release_id: str
    """ID of the release."""

    status: Literal["drifted", "staged", "queued", "deployed", "archived", "failed", "retrying"]
    """
    This status merges the 'activity_status' and 'error_status' fields, with error
    states taking precedence over activity states when errors are present. For
    example, if the activity status is 'deployed' but the error status is 'failed',
    the status is 'failed'. However, if the error status is 'none' and the activity
    status is 'deployed', the status is 'deployed'.
    """

    target_status: Literal["staged", "deployed", "archived"]
    """Desired state of the deployment.

    - Staged: is ready to be deployed
    - Deployed: all config instances part of the deployment are available for
      consumption on the device
    - Archived: the deployment is available for historical reference but cannot be
      deployed and is not active on the device
    """

    updated_at: datetime
    """Timestamp of when the device release was last updated."""

    config_instances: Optional[List[ConfigInstance]] = None
    """
    Expand the config instances using 'expand=config_instances' in the query string.
    """

    device: Optional[Device] = None
    """Expand the device using 'expand=device' in the query string."""

    release: Optional[Release] = None
    """Expand the release using 'expand=release' in the query string."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DeviceIssueActivationTokenParams"]


class DeviceIssueActivationTokenParams(TypedDict, total=False):
    allow_reactivation: bool
    """Whether this token can reactivate already activated devices.

    If false, the token is unable to activate devices which are already activated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Principal"]


class Principal(BaseModel):
    id: str
    """ID of the principal."""

    name: str
    """The display name of the principal."""

    object: Literal["principal"]
    """The object type, which is always `principal`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DeviceUpdateParams"]


class DeviceUpdateParams(TypedDict, total=False):
    name: str
    """The new name of the device.

    If excluded from the request, the device name is not updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["GitCommitRefParam"]


class GitCommitRefParam(TypedDict, total=False):
    """A reference to a git commit.

    At least one of `id` or `sha` must be provided. When both are provided, `id` takes precedence and `sha` is ignored.
    """

    id: str
    """ID of the git commit. Takes precedence over `sha` when both are provided."""

    sha: str
    """The SHA hash of the git commit. Used only when `id` is not provided."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import config_type_list_params, config_type_create_params, config_type_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.config_type import ConfigType
from ..types.config_type_list_response import ConfigTypeListResponse

__all__ = ["ConfigTypesResource", "AsyncConfigTypesResource"]


class ConfigTypesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConfigTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return ConfigTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return ConfigTypesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        slug: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Create a new config type.

        Args:
          name: The name of the config type.

          slug: An immutable, code-friendly name for the config type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/config_types",
            body=maybe_transform(
                {
                    "name": name,
                    "slug": slug,
                },
                config_type_create_params.ConfigTypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    def retrieve(
        self,
        config_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Retrieve a config type by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_type_id:
            raise ValueError(f"Expected a non-empty value for `config_type_id` but received {config_type_id!r}")
        return self._get(
            f"/config_types/{config_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    def update(
        self,
        config_type_id: str,
        *,
        name: str | Omit = omit,
        slug: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Update a config type by ID.

        Args:
          name: The updated name of the config type.

          slug: The updated slug for the config type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_type_id:
            raise ValueError(f"Expected a non-empty value for `config_type_id` but received {config_type_id!r}")
        return self._patch(
            f"/config_types/{config_type_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "slug": slug,
                },
                config_type_update_params.ConfigTypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        expand: List[Literal["total_count"]] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        slug: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigTypeListResponse:
        """
        List config types.

        Args:
          id: The config type IDs to filter by.

          expand: Fields to expand on each config type in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          name: The config type names to filter by.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the config type results.

          slug: The config type slugs to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/config_types",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "name": name,
                        "offset": offset,
                        "order_by": order_by,
                        "slug": slug,
                    },
                    config_type_list_params.ConfigTypeListParams,
                ),
            ),
            cast_to=ConfigTypeListResponse,
        )


class AsyncConfigTypesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConfigTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return AsyncConfigTypesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        slug: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Create a new config type.

        Args:
          name: The name of the config type.

          slug: An immutable, code-friendly name for the config type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/config_types",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "slug": slug,
                },
                config_type_create_params.ConfigTypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    async def retrieve(
        self,
        config_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Retrieve a config type by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_type_id:
            raise ValueError(f"Expected a non-empty value for `config_type_id` but received {config_type_id!r}")
        return await self._get(
            f"/config_types/{config_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    async def update(
        self,
        config_type_id: str,
        *,
        name: str | Omit = omit,
        slug: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigType:
        """
        Update a config type by ID.

        Args:
          name: The updated name of the config type.

          slug: The updated slug for the config type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_type_id:
            raise ValueError(f"Expected a non-empty value for `config_type_id` but received {config_type_id!r}")
        return await self._patch(
            f"/config_types/{config_type_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "slug": slug,
                },
                config_type_update_params.ConfigTypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigType,
        )

    async def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        expand: List[Literal["total_count"]] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        slug: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConfigTypeListResponse:
        """
        List config types.

        Args:
          id: The config type IDs to filter by.

          expand: Fields to expand on each config type in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          name: The config type names to filter by.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the config type results.

          slug: The config type slugs to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/config_types",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "name": name,
                        "offset": offset,
                        "order_by": order_by,
                        "slug": slug,
                    },
                    config_type_list_params.ConfigTypeListParams,
                ),
            ),
            cast_to=ConfigTypeListResponse,
        )


class ConfigTypesResourceWithRawResponse:
    def __init__(self, config_types: ConfigTypesResource) -> None:
        self._config_types = config_types

        self.create = to_raw_response_wrapper(
            config_types.create,
        )
        self.retrieve = to_raw_response_wrapper(
            config_types.retrieve,
        )
        self.update = to_raw_response_wrapper(
            config_types.update,
        )
        self.list = to_raw_response_wrapper(
            config_types.list,
        )


class AsyncConfigTypesResourceWithRawResponse:
    def __init__(self, config_types: AsyncConfigTypesResource) -> None:
        self._config_types = config_types

        self.create = async_to_raw_response_wrapper(
            config_types.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            config_types.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            config_types.update,
        )
        self.list = async_to_raw_response_wrapper(
            config_types.list,
        )


class ConfigTypesResourceWithStreamingResponse:
    def __init__(self, config_types: ConfigTypesResource) -> None:
        self._config_types = config_types

        self.create = to_streamed_response_wrapper(
            config_types.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            config_types.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            config_types.update,
        )
        self.list = to_streamed_response_wrapper(
            config_types.list,
        )


class AsyncConfigTypesResourceWithStreamingResponse:
    def __init__(self, config_types: AsyncConfigTypesResource) -> None:
        self._config_types = config_types

        self.create = async_to_streamed_response_wrapper(
            config_types.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            config_types.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            config_types.update,
        )
        self.list = async_to_streamed_response_wrapper(
            config_types.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConfigType"]


class ConfigType(BaseModel):
    id: str
    """ID of the config type."""

    created_at: datetime
    """Timestamp of when the config type was created."""

    name: str
    """Name of the config type."""

    object: Literal["config_type"]
    """The object type, which is always `config_type`."""

    slug: str
    """An immutable, code-friendly name for the config type."""

    updated_at: datetime
    """Timestamp of when the config type was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .config_schema import ConfigSchema

__all__ = ["Release"]


class Release(BaseModel):
    id: str
    """ID of the release."""

    created_at: datetime
    """Timestamp of when the release was created."""

    git_commit_id: Optional[str] = None
    """The ID of the git commit associated with this release."""

    object: Literal["release"]
    """The object type, which is always `release`."""

    updated_at: datetime
    """Timestamp of when the release was last updated."""

    version: str
    """The version of the release."""

    config_schemas: Optional[List[ConfigSchema]] = None
    """Expand the config schemas using 'expand=config_schemas' in the query string."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GitCommitCreateParams"]


class GitCommitCreateParams(TypedDict, total=False):
    message: Required[str]
    """The commit message."""

    origin: Required[str]
    """The URL of the git repository origin."""

    sha: Required[str]
    """The SHA hash of the git commit."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import git_commit_list_params, git_commit_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.git_commit import GitCommit
from ..types.git_commit_list import GitCommitList

__all__ = ["GitCommitsResource", "AsyncGitCommitsResource"]


class GitCommitsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GitCommitsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return GitCommitsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GitCommitsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return GitCommitsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        message: str,
        origin: str,
        sha: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommit:
        """
        Register a git commit.

        Args:
          message: The commit message.

          origin: The URL of the git repository origin.

          sha: The SHA hash of the git commit.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/git_commits",
            body=maybe_transform(
                {
                    "message": message,
                    "origin": origin,
                    "sha": sha,
                },
                git_commit_create_params.GitCommitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitCommit,
        )

    def retrieve(
        self,
        git_commit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommit:
        """
        Retrieve a git commit by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not git_commit_id:
            raise ValueError(f"Expected a non-empty value for `git_commit_id` but received {git_commit_id!r}")
        return self._get(
            f"/git_commits/{git_commit_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitCommit,
        )

    def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        expand: List[Literal["total_count"]] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        sha: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommitList:
        """
        List git commits.

        Args:
          id: The git commit IDs to filter by.

          expand: Fields to expand on each git commit in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the git commit results.

          sha: The git commit SHAs to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/git_commits",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "sha": sha,
                    },
                    git_commit_list_params.GitCommitListParams,
                ),
            ),
            cast_to=GitCommitList,
        )


class AsyncGitCommitsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGitCommitsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncGitCommitsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGitCommitsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return AsyncGitCommitsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        message: str,
        origin: str,
        sha: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommit:
        """
        Register a git commit.

        Args:
          message: The commit message.

          origin: The URL of the git repository origin.

          sha: The SHA hash of the git commit.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/git_commits",
            body=await async_maybe_transform(
                {
                    "message": message,
                    "origin": origin,
                    "sha": sha,
                },
                git_commit_create_params.GitCommitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitCommit,
        )

    async def retrieve(
        self,
        git_commit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommit:
        """
        Retrieve a git commit by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not git_commit_id:
            raise ValueError(f"Expected a non-empty value for `git_commit_id` but received {git_commit_id!r}")
        return await self._get(
            f"/git_commits/{git_commit_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GitCommit,
        )

    async def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        expand: List[Literal["total_count"]] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        sha: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GitCommitList:
        """
        List git commits.

        Args:
          id: The git commit IDs to filter by.

          expand: Fields to expand on each git commit in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the git commit results.

          sha: The git commit SHAs to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/git_commits",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "sha": sha,
                    },
                    git_commit_list_params.GitCommitListParams,
                ),
            ),
            cast_to=GitCommitList,
        )


class GitCommitsResourceWithRawResponse:
    def __init__(self, git_commits: GitCommitsResource) -> None:
        self._git_commits = git_commits

        self.create = to_raw_response_wrapper(
            git_commits.create,
        )
        self.retrieve = to_raw_response_wrapper(
            git_commits.retrieve,
        )
        self.list = to_raw_response_wrapper(
            git_commits.list,
        )


class AsyncGitCommitsResourceWithRawResponse:
    def __init__(self, git_commits: AsyncGitCommitsResource) -> None:
        self._git_commits = git_commits

        self.create = async_to_raw_response_wrapper(
            git_commits.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            git_commits.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            git_commits.list,
        )


class GitCommitsResourceWithStreamingResponse:
    def __init__(self, git_commits: GitCommitsResource) -> None:
        self._git_commits = git_commits

        self.create = to_streamed_response_wrapper(
            git_commits.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            git_commits.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            git_commits.list,
        )


class AsyncGitCommitsResourceWithStreamingResponse:
    def __init__(self, git_commits: AsyncGitCommitsResource) -> None:
        self._git_commits = git_commits

        self.create = async_to_streamed_response_wrapper(
            git_commits.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            git_commits.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            git_commits.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from miru_platform_sdk import Miru, AsyncMiru
from miru_platform_sdk.types import GitCommit, GitCommitList

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGitCommits:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Miru) -> None:
        git_commit = client.git_commits.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        )
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Miru) -> None:
        response = client.git_commits.with_raw_response.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = response.parse()
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Miru) -> None:
        with client.git_commits.with_streaming_response.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = response.parse()
            assert_matches_type(GitCommit, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Miru) -> None:
        git_commit = client.git_commits.retrieve(
            "git_commit_123",
        )
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Miru) -> None:
        response = client.git_commits.with_raw_response.retrieve(
            "git_commit_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = response.parse()
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Miru) -> None:
        with client.git_commits.with_streaming_response.retrieve(
            "git_commit_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = response.parse()
            assert_matches_type(GitCommit, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `git_commit_id` but received ''"):
            client.git_commits.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Miru) -> None:
        git_commit = client.git_commits.list()
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Miru) -> None:
        git_commit = client.git_commits.list(
            id=["git_cmt_123"],
            expand=["total_count"],
            limit=10,
            offset=0,
            order_by=["created_at:desc"],
            sha=["a1b2c3d4e5f6"],
        )
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Miru) -> None:
        response = client.git_commits.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = response.parse()
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Miru) -> None:
        with client.git_commits.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = response.parse()
            assert_matches_type(GitCommitList, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncGitCommits:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncMiru) -> None:
        git_commit = await async_client.git_commits.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        )
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMiru) -> None:
        response = await async_client.git_commits.with_raw_response.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = await response.parse()
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMiru) -> None:
        async with async_client.git_commits.with_streaming_response.create(
            message="Update config schema for robot1",
            origin="https://github.com/robot1/robot1.git",
            sha="a1b2c3d4e5f6a7b8c9d0e1f2a3b4c5d6e7f8a9b0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = await response.parse()
            assert_matches_type(GitCommit, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMiru) -> None:
        git_commit = await async_client.git_commits.retrieve(
            "git_commit_123",
        )
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMiru) -> None:
        response = await async_client.git_commits.with_raw_response.retrieve(
            "git_commit_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = await response.parse()
        assert_matches_type(GitCommit, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMiru) -> None:
        async with async_client.git_commits.with_streaming_response.retrieve(
            "git_commit_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = await response.parse()
            assert_matches_type(GitCommit, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `git_commit_id` but received ''"):
            await async_client.git_commits.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMiru) -> None:
        git_commit = await async_client.git_commits.list()
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMiru) -> None:
        git_commit = await async_client.git_commits.list(
            id=["git_cmt_123"],
            expand=["total_count"],
            limit=10,
            offset=0,
            order_by=["created_at:desc"],
            sha=["a1b2c3d4e5f6"],
        )
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMiru) -> None:
        response = await async_client.git_commits.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        git_commit = await response.parse()
        assert_matches_type(GitCommitList, git_commit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMiru) -> None:
        async with async_client.git_commits.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            git_commit = await response.parse()
            assert_matches_type(GitCommitList, git_commit, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .git_commit import GitCommit
from .shared.paginated_list import PaginatedList

__all__ = ["GitCommitList"]


class GitCommitList(PaginatedList):
    data: List[GitCommit]
    """The list of git commits."""



# Note: initially copied from https://github.com/florimondmanca/httpx-sse/blob/master/src/httpx_sse/_decoders.py
from __future__ import annotations

import json
import inspect
from types import TracebackType
from typing import TYPE_CHECKING, Any, Generic, TypeVar, Iterator, Optional, AsyncIterator, cast
from typing_extensions import Self, Protocol, TypeGuard, override, get_origin, runtime_checkable

import httpx

from ._utils import extract_type_var_from_base

if TYPE_CHECKING:
    from ._client import Miru, AsyncMiru
    from ._models import FinalRequestOptions


_T = TypeVar("_T")


class Stream(Generic[_T]):
    """Provides the core interface to iterate over a synchronous stream response."""

    response: httpx.Response
    _options: Optional[FinalRequestOptions] = None
    _decoder: SSEBytesDecoder

    def __init__(
        self,
        *,
        cast_to: type[_T],
        response: httpx.Response,
        client: Miru,
        options: Optional[FinalRequestOptions] = None,
    ) -> None:
        self.response = response
        self._cast_to = cast_to
        self._client = client
        self._options = options
        self._decoder = client._make_sse_decoder()
        self._iterator = self.__stream__()

    def __next__(self) -> _T:
        return self._iterator.__next__()

    def __iter__(self) -> Iterator[_T]:
        for item in self._iterator:
            yield item

    def _iter_events(self) -> Iterator[ServerSentEvent]:
        yield from self._decoder.iter_bytes(self.response.iter_bytes())

    def __stream__(self) -> Iterator[_T]:
        cast_to = cast(Any, self._cast_to)
        response = self.response
        process_data = self._client._process_response_data
        iterator = self._iter_events()

        try:
            for sse in iterator:
                yield process_data(data=sse.json(), cast_to=cast_to, response=response)
        finally:
            # Ensure the response is closed even if the consumer doesn't read all data
            response.close()

    def __enter__(self) -> Self:
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self.close()

    def close(self) -> None:
        """
        Close the response and release the connection.

        Automatically called if the response body is read to completion.
        """
        self.response.close()


class AsyncStream(Generic[_T]):
    """Provides the core interface to iterate over an asynchronous stream response."""

    response: httpx.Response
    _options: Optional[FinalRequestOptions] = None
    _decoder: SSEDecoder | SSEBytesDecoder

    def __init__(
        self,
        *,
        cast_to: type[_T],
        response: httpx.Response,
        client: AsyncMiru,
        options: Optional[FinalRequestOptions] = None,
    ) -> None:
        self.response = response
        self._cast_to = cast_to
        self._client = client
        self._options = options
        self._decoder = client._make_sse_decoder()
        self._iterator = self.__stream__()

    async def __anext__(self) -> _T:
        return await self._iterator.__anext__()

    async def __aiter__(self) -> AsyncIterator[_T]:
        async for item in self._iterator:
            yield item

    async def _iter_events(self) -> AsyncIterator[ServerSentEvent]:
        async for sse in self._decoder.aiter_bytes(self.response.aiter_bytes()):
            yield sse

    async def __stream__(self) -> AsyncIterator[_T]:
        cast_to = cast(Any, self._cast_to)
        response = self.response
        process_data = self._client._process_response_data
        iterator = self._iter_events()

        try:
            async for sse in iterator:
                yield process_data(data=sse.json(), cast_to=cast_to, response=response)
        finally:
            # Ensure the response is closed even if the consumer doesn't read all data
            await response.aclose()

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        await self.close()

    async def close(self) -> None:
        """
        Close the response and release the connection.

        Automatically called if the response body is read to completion.
        """
        await self.response.aclose()


class ServerSentEvent:
    def __init__(
        self,
        *,
        event: str | None = None,
        data: str | None = None,
        id: str | None = None,
        retry: int | None = None,
    ) -> None:
        if data is None:
            data = ""

        self._id = id
        self._data = data
        self._event = event or None
        self._retry = retry

    @property
    def event(self) -> str | None:
        return self._event

    @property
    def id(self) -> str | None:
        return self._id

    @property
    def retry(self) -> int | None:
        return self._retry

    @property
    def data(self) -> str:
        return self._data

    def json(self) -> Any:
        return json.loads(self.data)

    @override
    def __repr__(self) -> str:
        return f"ServerSentEvent(event={self.event}, data={self.data}, id={self.id}, retry={self.retry})"


class SSEDecoder:
    _data: list[str]
    _event: str | None
    _retry: int | None
    _last_event_id: str | None

    def __init__(self) -> None:
        self._event = None
        self._data = []
        self._last_event_id = None
        self._retry = None

    def iter_bytes(self, iterator: Iterator[bytes]) -> Iterator[ServerSentEvent]:
        """Given an iterator that yields raw binary data, iterate over it & yield every event encountered"""
        for chunk in self._iter_chunks(iterator):
            # Split before decoding so splitlines() only uses \r and \n
            for raw_line in chunk.splitlines():
                line = raw_line.decode("utf-8")
                sse = self.decode(line)
                if sse:
                    yield sse

    def _iter_chunks(self, iterator: Iterator[bytes]) -> Iterator[bytes]:
        """Given an iterator that yields raw binary data, iterate over it and yield individual SSE chunks"""
        data = b""
        for chunk in iterator:
            for line in chunk.splitlines(keepends=True):
                data += line
                if data.endswith((b"\r\r", b"\n\n", b"\r\n\r\n")):
                    yield data
                    data = b""
        if data:
            yield data

    async def aiter_bytes(self, iterator: AsyncIterator[bytes]) -> AsyncIterator[ServerSentEvent]:
        """Given an iterator that yields raw binary data, iterate over it & yield every event encountered"""
        async for chunk in self._aiter_chunks(iterator):
            # Split before decoding so splitlines() only uses \r and \n
            for raw_line in chunk.splitlines():
                line = raw_line.decode("utf-8")
                sse = self.decode(line)
                if sse:
                    yield sse

    async def _aiter_chunks(self, iterator: AsyncIterator[bytes]) -> AsyncIterator[bytes]:
        """Given an iterator that yields raw binary data, iterate over it and yield individual SSE chunks"""
        data = b""
        async for chunk in iterator:
            for line in chunk.splitlines(keepends=True):
                data += line
                if data.endswith((b"\r\r", b"\n\n", b"\r\n\r\n")):
                    yield data
                    data = b""
        if data:
            yield data

    def decode(self, line: str) -> ServerSentEvent | None:
        # See: https://html.spec.whatwg.org/multipage/server-sent-events.html#event-stream-interpretation  # noqa: E501

        if not line:
            if not self._event and not self._data and not self._last_event_id and self._retry is None:
                return None

            sse = ServerSentEvent(
                event=self._event,
                data="\n".join(self._data),
                id=self._last_event_id,
                retry=self._retry,
            )

            # NOTE: as per the SSE spec, do not reset last_event_id.
            self._event = None
            self._data = []
            self._retry = None

            return sse

        if line.startswith(":"):
            return None

        fieldname, _, value = line.partition(":")

        if value.startswith(" "):
            value = value[1:]

        if fieldname == "event":
            self._event = value
        elif fieldname == "data":
            self._data.append(value)
        elif fieldname == "id":
            if "\0" in value:
                pass
            else:
                self._last_event_id = value
        elif fieldname == "retry":
            try:
                self._retry = int(value)
            except (TypeError, ValueError):
                pass
        else:
            pass  # Field is ignored.

        return None


@runtime_checkable
class SSEBytesDecoder(Protocol):
    def iter_bytes(self, iterator: Iterator[bytes]) -> Iterator[ServerSentEvent]:
        """Given an iterator that yields raw binary data, iterate over it & yield every event encountered"""
        ...

    def aiter_bytes(self, iterator: AsyncIterator[bytes]) -> AsyncIterator[ServerSentEvent]:
        """Given an async iterator that yields raw binary data, iterate over it & yield every event encountered"""
        ...


def is_stream_class_type(typ: type) -> TypeGuard[type[Stream[object]] | type[AsyncStream[object]]]:
    """TypeGuard for determining whether or not the given type is a subclass of `Stream` / `AsyncStream`"""
    origin = get_origin(typ) or typ
    return inspect.isclass(origin) and issubclass(origin, (Stream, AsyncStream))


def extract_stream_chunk_type(
    stream_cls: type,
    *,
    failure_message: str | None = None,
) -> type:
    """Given a type like `Stream[T]`, returns the generic type variable `T`.

    This also handles the case where a concrete subclass is given, e.g.
    ```py
    class MyStream(Stream[bytes]):
        ...

    extract_stream_chunk_type(MyStream) -> bytes
    ```
    """
    from ._base_client import Stream, AsyncStream

    return extract_type_var_from_base(
        stream_cls,
        index=0,
        generic_bases=cast("tuple[type, ...]", (Stream, AsyncStream)),
        failure_message=failure_message,
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .release import Release
from .shared.paginated_list import PaginatedList

__all__ = ["ReleaseList"]


class ReleaseList(PaginatedList):
    data: List[Release]
    """The list of releases."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["DeploymentListParams"]


class DeploymentListParams(TypedDict, total=False):
    id: SequenceNotStr[str]
    """The deployment IDs to filter by."""

    activity_status: List[Literal["drifted", "staged", "queued", "deployed", "archived"]]
    """The deployment activity statuses to filter by."""

    device_id: SequenceNotStr[str]
    """The deployment device IDs to filter by."""

    error_status: List[Literal["none", "failed", "retrying"]]
    """The deployment error statuses to filter by."""

    expand: List[Literal["total_count", "device", "release", "config_instances"]]
    """Fields to expand on each deployment in the list."""

    limit: int
    """The maximum number of items to return.

    A limit of 15 with an offset of 0 returns items 1-15.
    """

    offset: int
    """The offset of the items to return.

    An offset of 10 with a limit of 10 returns items 11-20.
    """

    order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]]
    """Sort order for the deployment results."""

    release_id: SequenceNotStr[str]
    """The deployment release IDs to filter by."""

    target_status: List[Literal["staged", "deployed", "archived"]]
    """The deployment target statuses to filter by."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["GitCommit"]


class GitCommit(BaseModel):
    id: str
    """ID of the git commit."""

    commit_url: str
    """The URL of the git commit."""

    created_at: datetime
    """Timestamp of when the git commit was created."""

    message: str
    """The commit message."""

    object: Literal["git_commit"]
    """The object type, which is always `git_commit`."""

    repository_name: str
    """The name of the git repository."""

    repository_owner: str
    """The owner of the git repository."""

    repository_type: Literal["github", "gitlab", "bitbucket"]

    repository_url: str
    """The URL of the git repository."""

    sha: str
    """The SHA hash of the git commit."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["ConfigTypeListParams"]


class ConfigTypeListParams(TypedDict, total=False):
    id: SequenceNotStr[str]
    """The config type IDs to filter by."""

    expand: List[Literal["total_count"]]
    """Fields to expand on each config type in the list."""

    limit: int
    """The maximum number of items to return.

    A limit of 15 with an offset of 0 returns items 1-15.
    """

    name: SequenceNotStr[str]
    """The config type names to filter by."""

    offset: int
    """The offset of the items to return.

    An offset of 10 with a limit of 10 returns items 11-20.
    """

    order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]]
    """Sort order for the config type results."""

    slug: SequenceNotStr[str]
    """The config type slugs to filter by."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .config_schema import ConfigSchema
from .shared.paginated_list import PaginatedList

__all__ = ["ConfigSchemaList"]


class ConfigSchemaList(PaginatedList):
    data: List[ConfigSchema]
    """The list of config schemas."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DevicePingResponse"]


class DevicePingResponse(BaseModel):
    code: Literal["success", "timeout"]
    """The result of the ping operation."""

    round_trip_nanos: Optional[int] = None
    """
    The round trip time in nanoseconds to send the ping to the device and receive
    the response. Only present when `code` is `success`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import (
    deployment_list_params,
    deployment_create_params,
    deployment_deploy_params,
    deployment_archive_params,
    deployment_retrieve_params,
    deployment_list_drifts_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.deployment import Deployment
from ..types.deployment_list import DeploymentList

__all__ = ["DeploymentsResource", "AsyncDeploymentsResource"]


class DeploymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return DeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return DeploymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        config_instance_ids: SequenceNotStr[str],
        description: str,
        device_id: str,
        release_id: str,
        target_status: Literal["staged", "deployed"],
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        parent_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Stage or deploy a new deployment.

        Args:
          config_instance_ids: The IDs of the config instances to deploy. A deployment must have exactly one
              config instance for each config schema in the deployment's release.

          description: The description of the deployment.

          device_id: The ID of the device that the deployment is being created for.

          release_id: The release ID which this deployment adheres to.

          target_status: Desired state of the deployment.

              - Staged: ready for deployment. Deployments can only be staged if their release
                is not the current release for the device.
              - Deployed: deployed to the device. Deployments can only be deployed if their
                release is the device's current release.

          expand: Fields to expand on the deployment resource.

          parent_id: The ID of the deployment that this deployment was patched from.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/deployments",
            body=maybe_transform(
                {
                    "config_instance_ids": config_instance_ids,
                    "description": description,
                    "device_id": device_id,
                    "release_id": release_id,
                    "target_status": target_status,
                    "parent_id": parent_id,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, deployment_create_params.DeploymentCreateParams),
            ),
            cast_to=Deployment,
        )

    def retrieve(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Retrieve a deployment by its ID.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._get(
            f"/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, deployment_retrieve_params.DeploymentRetrieveParams),
            ),
            cast_to=Deployment,
        )

    def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        activity_status: List[Literal["drifted", "staged", "queued", "deployed", "archived"]] | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        error_status: List[Literal["none", "failed", "retrying"]] | Omit = omit,
        expand: List[Literal["total_count", "device", "release", "config_instances"]] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        release_id: SequenceNotStr[str] | Omit = omit,
        target_status: List[Literal["staged", "deployed", "archived"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentList:
        """
        List all deployments.

        Args:
          id: The deployment IDs to filter by.

          activity_status: The deployment activity statuses to filter by.

          device_id: The deployment device IDs to filter by.

          error_status: The deployment error statuses to filter by.

          expand: Fields to expand on each deployment in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the deployment results.

          release_id: The deployment release IDs to filter by.

          target_status: The deployment target statuses to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/deployments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "activity_status": activity_status,
                        "device_id": device_id,
                        "error_status": error_status,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "release_id": release_id,
                        "target_status": target_status,
                    },
                    deployment_list_params.DeploymentListParams,
                ),
            ),
            cast_to=DeploymentList,
        )

    def archive(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Archive staged or drifted deployments.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._post(
            f"/deployments/{deployment_id}/archive",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, deployment_archive_params.DeploymentArchiveParams),
            ),
            cast_to=Deployment,
        )

    def deploy(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Trigger a deployment to its target device.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._post(
            f"/deployments/{deployment_id}/deploy",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, deployment_deploy_params.DeploymentDeployParams),
            ),
            cast_to=Deployment,
        )

    def list_drifts(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentList:
        """
        List the deployments which have been deployed since this deployment was staged.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._get(
            f"/deployments/{deployment_id}/drifts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, deployment_list_drifts_params.DeploymentListDriftsParams),
            ),
            cast_to=DeploymentList,
        )


class AsyncDeploymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/mirurobotics/python-platform-sdk#with_streaming_response
        """
        return AsyncDeploymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        config_instance_ids: SequenceNotStr[str],
        description: str,
        device_id: str,
        release_id: str,
        target_status: Literal["staged", "deployed"],
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        parent_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Stage or deploy a new deployment.

        Args:
          config_instance_ids: The IDs of the config instances to deploy. A deployment must have exactly one
              config instance for each config schema in the deployment's release.

          description: The description of the deployment.

          device_id: The ID of the device that the deployment is being created for.

          release_id: The release ID which this deployment adheres to.

          target_status: Desired state of the deployment.

              - Staged: ready for deployment. Deployments can only be staged if their release
                is not the current release for the device.
              - Deployed: deployed to the device. Deployments can only be deployed if their
                release is the device's current release.

          expand: Fields to expand on the deployment resource.

          parent_id: The ID of the deployment that this deployment was patched from.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/deployments",
            body=await async_maybe_transform(
                {
                    "config_instance_ids": config_instance_ids,
                    "description": description,
                    "device_id": device_id,
                    "release_id": release_id,
                    "target_status": target_status,
                    "parent_id": parent_id,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"expand": expand}, deployment_create_params.DeploymentCreateParams),
            ),
            cast_to=Deployment,
        )

    async def retrieve(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Retrieve a deployment by its ID.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._get(
            f"/deployments/{deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, deployment_retrieve_params.DeploymentRetrieveParams
                ),
            ),
            cast_to=Deployment,
        )

    async def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        activity_status: List[Literal["drifted", "staged", "queued", "deployed", "archived"]] | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        error_status: List[Literal["none", "failed", "retrying"]] | Omit = omit,
        expand: List[Literal["total_count", "device", "release", "config_instances"]] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        order_by: List[Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]] | Omit = omit,
        release_id: SequenceNotStr[str] | Omit = omit,
        target_status: List[Literal["staged", "deployed", "archived"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentList:
        """
        List all deployments.

        Args:
          id: The deployment IDs to filter by.

          activity_status: The deployment activity statuses to filter by.

          device_id: The deployment device IDs to filter by.

          error_status: The deployment error statuses to filter by.

          expand: Fields to expand on each deployment in the list.

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          order_by: Sort order for the deployment results.

          release_id: The deployment release IDs to filter by.

          target_status: The deployment target statuses to filter by.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/deployments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "activity_status": activity_status,
                        "device_id": device_id,
                        "error_status": error_status,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "release_id": release_id,
                        "target_status": target_status,
                    },
                    deployment_list_params.DeploymentListParams,
                ),
            ),
            cast_to=DeploymentList,
        )

    async def archive(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Archive staged or drifted deployments.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._post(
            f"/deployments/{deployment_id}/archive",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, deployment_archive_params.DeploymentArchiveParams
                ),
            ),
            cast_to=Deployment,
        )

    async def deploy(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Trigger a deployment to its target device.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._post(
            f"/deployments/{deployment_id}/deploy",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"expand": expand}, deployment_deploy_params.DeploymentDeployParams),
            ),
            cast_to=Deployment,
        )

    async def list_drifts(
        self,
        deployment_id: str,
        *,
        expand: List[Literal["device", "release", "config_instances"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentList:
        """
        List the deployments which have been deployed since this deployment was staged.

        Args:
          expand: Fields to expand on the deployment resource.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._get(
            f"/deployments/{deployment_id}/drifts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, deployment_list_drifts_params.DeploymentListDriftsParams
                ),
            ),
            cast_to=DeploymentList,
        )


class DeploymentsResourceWithRawResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_raw_response_wrapper(
            deployments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            deployments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            deployments.list,
        )
        self.archive = to_raw_response_wrapper(
            deployments.archive,
        )
        self.deploy = to_raw_response_wrapper(
            deployments.deploy,
        )
        self.list_drifts = to_raw_response_wrapper(
            deployments.list_drifts,
        )


class AsyncDeploymentsResourceWithRawResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_raw_response_wrapper(
            deployments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            deployments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            deployments.list,
        )
        self.archive = async_to_raw_response_wrapper(
            deployments.archive,
        )
        self.deploy = async_to_raw_response_wrapper(
            deployments.deploy,
        )
        self.list_drifts = async_to_raw_response_wrapper(
            deployments.list_drifts,
        )


class DeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_streamed_response_wrapper(
            deployments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            deployments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            deployments.list,
        )
        self.archive = to_streamed_response_wrapper(
            deployments.archive,
        )
        self.deploy = to_streamed_response_wrapper(
            deployments.deploy,
        )
        self.list_drifts = to_streamed_response_wrapper(
            deployments.list_drifts,
        )


class AsyncDeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_streamed_response_wrapper(
            deployments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            deployments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            deployments.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            deployments.archive,
        )
        self.deploy = async_to_streamed_response_wrapper(
            deployments.deploy,
        )
        self.list_drifts = async_to_streamed_response_wrapper(
            deployments.list_drifts,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from miru_platform_sdk import Miru, AsyncMiru
from miru_platform_sdk.types import (
    Deployment,
    DeploymentList,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDeployments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Miru) -> None:
        deployment = client.deployments.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
            expand=["device"],
            parent_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Miru) -> None:
        deployment = client.deployments.retrieve(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.retrieve(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.retrieve(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.retrieve(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.deployments.with_raw_response.retrieve(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Miru) -> None:
        deployment = client.deployments.list()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.list(
            id=["dpl_123"],
            activity_status=["drifted"],
            device_id=["dvc_123"],
            error_status=["none"],
            expand=["device"],
            limit=10,
            offset=0,
            order_by=["created_at:desc"],
            release_id=["rls_123"],
            target_status=["staged"],
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(DeploymentList, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive(self, client: Miru) -> None:
        deployment = client.deployments.archive(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.archive(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_archive(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.archive(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_archive(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.archive(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_archive(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.deployments.with_raw_response.archive(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_deploy(self, client: Miru) -> None:
        deployment = client.deployments.deploy(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_deploy_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.deploy(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_deploy(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.deploy(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_deploy(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.deploy(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_deploy(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.deployments.with_raw_response.deploy(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_drifts(self, client: Miru) -> None:
        deployment = client.deployments.list_drifts(
            deployment_id="dpl_123",
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_drifts_with_all_params(self, client: Miru) -> None:
        deployment = client.deployments.list_drifts(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_drifts(self, client: Miru) -> None:
        response = client.deployments.with_raw_response.list_drifts(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_drifts(self, client: Miru) -> None:
        with client.deployments.with_streaming_response.list_drifts(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(DeploymentList, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_drifts(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.deployments.with_raw_response.list_drifts(
                deployment_id="",
            )


class TestAsyncDeployments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
            expand=["device"],
            parent_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.create(
            config_instance_ids=["cfg_inst_123"],
            description="Deployment for the motion control config instance",
            device_id="dvc_123",
            release_id="rls_123",
            target_status="staged",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.retrieve(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.retrieve(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.retrieve(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.retrieve(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.deployments.with_raw_response.retrieve(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.list()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.list(
            id=["dpl_123"],
            activity_status=["drifted"],
            device_id=["dvc_123"],
            error_status=["none"],
            expand=["device"],
            limit=10,
            offset=0,
            order_by=["created_at:desc"],
            release_id=["rls_123"],
            target_status=["staged"],
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(DeploymentList, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.archive(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.archive(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.archive(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.archive(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_archive(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.deployments.with_raw_response.archive(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_deploy(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.deploy(
            deployment_id="dpl_123",
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_deploy_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.deploy(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_deploy(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.deploy(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(Deployment, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_deploy(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.deploy(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(Deployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_deploy(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.deployments.with_raw_response.deploy(
                deployment_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_drifts(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.list_drifts(
            deployment_id="dpl_123",
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_drifts_with_all_params(self, async_client: AsyncMiru) -> None:
        deployment = await async_client.deployments.list_drifts(
            deployment_id="dpl_123",
            expand=["device"],
        )
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_drifts(self, async_client: AsyncMiru) -> None:
        response = await async_client.deployments.with_raw_response.list_drifts(
            deployment_id="dpl_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(DeploymentList, deployment, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_drifts(self, async_client: AsyncMiru) -> None:
        async with async_client.deployments.with_streaming_response.list_drifts(
            deployment_id="dpl_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(DeploymentList, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_drifts(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.deployments.with_raw_response.list_drifts(
                deployment_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import MiruError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        devices,
        releases,
        principal,
        deployments,
        git_commits,
        config_types,
        config_schemas,
        config_instances,
    )
    from .resources.devices import DevicesResource, AsyncDevicesResource
    from .resources.releases import ReleasesResource, AsyncReleasesResource
    from .resources.principal import PrincipalResource, AsyncPrincipalResource
    from .resources.deployments import DeploymentsResource, AsyncDeploymentsResource
    from .resources.git_commits import GitCommitsResource, AsyncGitCommitsResource
    from .resources.config_types import ConfigTypesResource, AsyncConfigTypesResource
    from .resources.config_schemas import ConfigSchemasResource, AsyncConfigSchemasResource
    from .resources.config_instances import ConfigInstancesResource, AsyncConfigInstancesResource

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Miru",
    "AsyncMiru",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "prod": "https://api.mirurobotics.com/beta",
    "uat": "https://uat.api.mirurobotics.com/beta",
    "staging": "https://staging.api.mirurobotics.com/beta",
    "local": "http://localhost:8080/beta",
}


class Miru(SyncAPIClient):
    # client options
    api_key: str
    miru_version: str

    _environment: Literal["prod", "uat", "staging", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        miru_version: str | None = None,
        environment: Literal["prod", "uat", "staging", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Miru client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MIRU_API_KEY`
        - `miru_version` from `MIRU_API_VERSION`
        """
        if api_key is None:
            api_key = os.environ.get("MIRU_API_KEY")
        if api_key is None:
            raise MiruError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MIRU_API_KEY environment variable"
            )
        self.api_key = api_key

        if miru_version is None:
            miru_version = os.environ.get("MIRU_API_VERSION") or "2026-03-09.tetons"
        self.miru_version = miru_version

        self._environment = environment

        base_url_env = os.environ.get("MIRU_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `MIRU_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "prod"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def config_instances(self) -> ConfigInstancesResource:
        from .resources.config_instances import ConfigInstancesResource

        return ConfigInstancesResource(self)

    @cached_property
    def config_schemas(self) -> ConfigSchemasResource:
        from .resources.config_schemas import ConfigSchemasResource

        return ConfigSchemasResource(self)

    @cached_property
    def config_types(self) -> ConfigTypesResource:
        from .resources.config_types import ConfigTypesResource

        return ConfigTypesResource(self)

    @cached_property
    def deployments(self) -> DeploymentsResource:
        from .resources.deployments import DeploymentsResource

        return DeploymentsResource(self)

    @cached_property
    def devices(self) -> DevicesResource:
        from .resources.devices import DevicesResource

        return DevicesResource(self)

    @cached_property
    def git_commits(self) -> GitCommitsResource:
        from .resources.git_commits import GitCommitsResource

        return GitCommitsResource(self)

    @cached_property
    def principal(self) -> PrincipalResource:
        from .resources.principal import PrincipalResource

        return PrincipalResource(self)

    @cached_property
    def releases(self) -> ReleasesResource:
        from .resources.releases import ReleasesResource

        return ReleasesResource(self)

    @cached_property
    def with_raw_response(self) -> MiruWithRawResponse:
        return MiruWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MiruWithStreamedResponse:
        return MiruWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"X-API-Key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "Miru-Version": self.miru_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        miru_version: str | None = None,
        environment: Literal["prod", "uat", "staging", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            miru_version=miru_version or self.miru_version,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncMiru(AsyncAPIClient):
    # client options
    api_key: str
    miru_version: str

    _environment: Literal["prod", "uat", "staging", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        miru_version: str | None = None,
        environment: Literal["prod", "uat", "staging", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncMiru client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MIRU_API_KEY`
        - `miru_version` from `MIRU_API_VERSION`
        """
        if api_key is None:
            api_key = os.environ.get("MIRU_API_KEY")
        if api_key is None:
            raise MiruError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MIRU_API_KEY environment variable"
            )
        self.api_key = api_key

        if miru_version is None:
            miru_version = os.environ.get("MIRU_API_VERSION") or "2026-03-09.tetons"
        self.miru_version = miru_version

        self._environment = environment

        base_url_env = os.environ.get("MIRU_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `MIRU_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "prod"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def config_instances(self) -> AsyncConfigInstancesResource:
        from .resources.config_instances import AsyncConfigInstancesResource

        return AsyncConfigInstancesResource(self)

    @cached_property
    def config_schemas(self) -> AsyncConfigSchemasResource:
        from .resources.config_schemas import AsyncConfigSchemasResource

        return AsyncConfigSchemasResource(self)

    @cached_property
    def config_types(self) -> AsyncConfigTypesResource:
        from .resources.config_types import AsyncConfigTypesResource

        return AsyncConfigTypesResource(self)

    @cached_property
    def deployments(self) -> AsyncDeploymentsResource:
        from .resources.deployments import AsyncDeploymentsResource

        return AsyncDeploymentsResource(self)

    @cached_property
    def devices(self) -> AsyncDevicesResource:
        from .resources.devices import AsyncDevicesResource

        return AsyncDevicesResource(self)

    @cached_property
    def git_commits(self) -> AsyncGitCommitsResource:
        from .resources.git_commits import AsyncGitCommitsResource

        return AsyncGitCommitsResource(self)

    @cached_property
    def principal(self) -> AsyncPrincipalResource:
        from .resources.principal import AsyncPrincipalResource

        return AsyncPrincipalResource(self)

    @cached_property
    def releases(self) -> AsyncReleasesResource:
        from .resources.releases import AsyncReleasesResource

        return AsyncReleasesResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncMiruWithRawResponse:
        return AsyncMiruWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMiruWithStreamedResponse:
        return AsyncMiruWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"X-API-Key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "Miru-Version": self.miru_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        miru_version: str | None = None,
        environment: Literal["prod", "uat", "staging", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            miru_version=miru_version or self.miru_version,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class MiruWithRawResponse:
    _client: Miru

    def __init__(self, client: Miru) -> None:
        self._client = client

    @cached_property
    def config_instances(self) -> config_instances.ConfigInstancesResourceWithRawResponse:
        from .resources.config_instances import ConfigInstancesResourceWithRawResponse

        return ConfigInstancesResourceWithRawResponse(self._client.config_instances)

    @cached_property
    def config_schemas(self) -> config_schemas.ConfigSchemasResourceWithRawResponse:
        from .resources.config_schemas import ConfigSchemasResourceWithRawResponse

        return ConfigSchemasResourceWithRawResponse(self._client.config_schemas)

    @cached_property
    def config_types(self) -> config_types.ConfigTypesResourceWithRawResponse:
        from .resources.config_types import ConfigTypesResourceWithRawResponse

        return ConfigTypesResourceWithRawResponse(self._client.config_types)

    @cached_property
    def deployments(self) -> deployments.DeploymentsResourceWithRawResponse:
        from .resources.deployments import DeploymentsResourceWithRawResponse

        return DeploymentsResourceWithRawResponse(self._client.deployments)

    @cached_property
    def devices(self) -> devices.DevicesResourceWithRawResponse:
        from .resources.devices import DevicesResourceWithRawResponse

        return DevicesResourceWithRawResponse(self._client.devices)

    @cached_property
    def git_commits(self) -> git_commits.GitCommitsResourceWithRawResponse:
        from .resources.git_commits import GitCommitsResourceWithRawResponse

        return GitCommitsResourceWithRawResponse(self._client.git_commits)

    @cached_property
    def principal(self) -> principal.PrincipalResourceWithRawResponse:
        from .resources.principal import PrincipalResourceWithRawResponse

        return PrincipalResourceWithRawResponse(self._client.principal)

    @cached_property
    def releases(self) -> releases.ReleasesResourceWithRawResponse:
        from .resources.releases import ReleasesResourceWithRawResponse

        return ReleasesResourceWithRawResponse(self._client.releases)


class AsyncMiruWithRawResponse:
    _client: AsyncMiru

    def __init__(self, client: AsyncMiru) -> None:
        self._client = client

    @cached_property
    def config_instances(self) -> config_instances.AsyncConfigInstancesResourceWithRawResponse:
        from .resources.config_instances import AsyncConfigInstancesResourceWithRawResponse

        return AsyncConfigInstancesResourceWithRawResponse(self._client.config_instances)

    @cached_property
    def config_schemas(self) -> config_schemas.AsyncConfigSchemasResourceWithRawResponse:
        from .resources.config_schemas import AsyncConfigSchemasResourceWithRawResponse

        return AsyncConfigSchemasResourceWithRawResponse(self._client.config_schemas)

    @cached_property
    def config_types(self) -> config_types.AsyncConfigTypesResourceWithRawResponse:
        from .resources.config_types import AsyncConfigTypesResourceWithRawResponse

        return AsyncConfigTypesResourceWithRawResponse(self._client.config_types)

    @cached_property
    def deployments(self) -> deployments.AsyncDeploymentsResourceWithRawResponse:
        from .resources.deployments import AsyncDeploymentsResourceWithRawResponse

        return AsyncDeploymentsResourceWithRawResponse(self._client.deployments)

    @cached_property
    def devices(self) -> devices.AsyncDevicesResourceWithRawResponse:
        from .resources.devices import AsyncDevicesResourceWithRawResponse

        return AsyncDevicesResourceWithRawResponse(self._client.devices)

    @cached_property
    def git_commits(self) -> git_commits.AsyncGitCommitsResourceWithRawResponse:
        from .resources.git_commits import AsyncGitCommitsResourceWithRawResponse

        return AsyncGitCommitsResourceWithRawResponse(self._client.git_commits)

    @cached_property
    def principal(self) -> principal.AsyncPrincipalResourceWithRawResponse:
        from .resources.principal import AsyncPrincipalResourceWithRawResponse

        return AsyncPrincipalResourceWithRawResponse(self._client.principal)

    @cached_property
    def releases(self) -> releases.AsyncReleasesResourceWithRawResponse:
        from .resources.releases import AsyncReleasesResourceWithRawResponse

        return AsyncReleasesResourceWithRawResponse(self._client.releases)


class MiruWithStreamedResponse:
    _client: Miru

    def __init__(self, client: Miru) -> None:
        self._client = client

    @cached_property
    def config_instances(self) -> config_instances.ConfigInstancesResourceWithStreamingResponse:
        from .resources.config_instances import ConfigInstancesResourceWithStreamingResponse

        return ConfigInstancesResourceWithStreamingResponse(self._client.config_instances)

    @cached_property
    def config_schemas(self) -> config_schemas.ConfigSchemasResourceWithStreamingResponse:
        from .resources.config_schemas import ConfigSchemasResourceWithStreamingResponse

        return ConfigSchemasResourceWithStreamingResponse(self._client.config_schemas)

    @cached_property
    def config_types(self) -> config_types.ConfigTypesResourceWithStreamingResponse:
        from .resources.config_types import ConfigTypesResourceWithStreamingResponse

        return ConfigTypesResourceWithStreamingResponse(self._client.config_types)

    @cached_property
    def deployments(self) -> deployments.DeploymentsResourceWithStreamingResponse:
        from .resources.deployments import DeploymentsResourceWithStreamingResponse

        return DeploymentsResourceWithStreamingResponse(self._client.deployments)

    @cached_property
    def devices(self) -> devices.DevicesResourceWithStreamingResponse:
        from .resources.devices import DevicesResourceWithStreamingResponse

        return DevicesResourceWithStreamingResponse(self._client.devices)

    @cached_property
    def git_commits(self) -> git_commits.GitCommitsResourceWithStreamingResponse:
        from .resources.git_commits import GitCommitsResourceWithStreamingResponse

        return GitCommitsResourceWithStreamingResponse(self._client.git_commits)

    @cached_property
    def principal(self) -> principal.PrincipalResourceWithStreamingResponse:
        from .resources.principal import PrincipalResourceWithStreamingResponse

        return PrincipalResourceWithStreamingResponse(self._client.principal)

    @cached_property
    def releases(self) -> releases.ReleasesResourceWithStreamingResponse:
        from .resources.releases import ReleasesResourceWithStreamingResponse

        return ReleasesResourceWithStreamingResponse(self._client.releases)


class AsyncMiruWithStreamedResponse:
    _client: AsyncMiru

    def __init__(self, client: AsyncMiru) -> None:
        self._client = client

    @cached_property
    def config_instances(self) -> config_instances.AsyncConfigInstancesResourceWithStreamingResponse:
        from .resources.config_instances import AsyncConfigInstancesResourceWithStreamingResponse

        return AsyncConfigInstancesResourceWithStreamingResponse(self._client.config_instances)

    @cached_property
    def config_schemas(self) -> config_schemas.AsyncConfigSchemasResourceWithStreamingResponse:
        from .resources.config_schemas import AsyncConfigSchemasResourceWithStreamingResponse

        return AsyncConfigSchemasResourceWithStreamingResponse(self._client.config_schemas)

    @cached_property
    def config_types(self) -> config_types.AsyncConfigTypesResourceWithStreamingResponse:
        from .resources.config_types import AsyncConfigTypesResourceWithStreamingResponse

        return AsyncConfigTypesResourceWithStreamingResponse(self._client.config_types)

    @cached_property
    def deployments(self) -> deployments.AsyncDeploymentsResourceWithStreamingResponse:
        from .resources.deployments import AsyncDeploymentsResourceWithStreamingResponse

        return AsyncDeploymentsResourceWithStreamingResponse(self._client.deployments)

    @cached_property
    def devices(self) -> devices.AsyncDevicesResourceWithStreamingResponse:
        from .resources.devices import AsyncDevicesResourceWithStreamingResponse

        return AsyncDevicesResourceWithStreamingResponse(self._client.devices)

    @cached_property
    def git_commits(self) -> git_commits.AsyncGitCommitsResourceWithStreamingResponse:
        from .resources.git_commits import AsyncGitCommitsResourceWithStreamingResponse

        return AsyncGitCommitsResourceWithStreamingResponse(self._client.git_commits)

    @cached_property
    def principal(self) -> principal.AsyncPrincipalResourceWithStreamingResponse:
        from .resources.principal import AsyncPrincipalResourceWithStreamingResponse

        return AsyncPrincipalResourceWithStreamingResponse(self._client.principal)

    @cached_property
    def releases(self) -> releases.AsyncReleasesResourceWithStreamingResponse:
        from .resources.releases import AsyncReleasesResourceWithStreamingResponse

        return AsyncReleasesResourceWithStreamingResponse(self._client.releases)


Client = Miru

AsyncClient = AsyncMiru



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ConfigTypeCreateParams"]


class ConfigTypeCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the config type."""

    slug: Required[str]
    """An immutable, code-friendly name for the config type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .device import Device as Device
from .shared import PaginatedList as PaginatedList
from .release import Release as Release
from .principal import Principal as Principal
from .deployment import Deployment as Deployment
from .git_commit import GitCommit as GitCommit
from .config_type import ConfigType as ConfigType
from .device_list import DeviceList as DeviceList
from .release_list import ReleaseList as ReleaseList
from .config_schema import ConfigSchema as ConfigSchema
from .config_instance import ConfigInstance as ConfigInstance
from .deployment_list import DeploymentList as DeploymentList
from .git_commit_list import GitCommitList as GitCommitList
from .schema_document import SchemaDocument as SchemaDocument
from .schema_language import SchemaLanguage as SchemaLanguage
from .instance_content import InstanceContent as InstanceContent
from .config_schema_list import ConfigSchemaList as ConfigSchemaList
from .device_list_params import DeviceListParams as DeviceListParams
from .device_ping_params import DevicePingParams as DevicePingParams
from .release_list_params import ReleaseListParams as ReleaseListParams
from .device_create_params import DeviceCreateParams as DeviceCreateParams
from .device_ping_response import DevicePingResponse as DevicePingResponse
from .device_update_params import DeviceUpdateParams as DeviceUpdateParams
from .git_commit_ref_param import GitCommitRefParam as GitCommitRefParam
from .release_create_params import ReleaseCreateParams as ReleaseCreateParams
from .deployment_list_params import DeploymentListParams as DeploymentListParams
from .git_commit_list_params import GitCommitListParams as GitCommitListParams
from .instance_content_param import InstanceContentParam as InstanceContentParam
from .config_type_list_params import ConfigTypeListParams as ConfigTypeListParams
from .release_retrieve_params import ReleaseRetrieveParams as ReleaseRetrieveParams
from .deployment_create_params import DeploymentCreateParams as DeploymentCreateParams
from .deployment_deploy_params import DeploymentDeployParams as DeploymentDeployParams
from .git_commit_create_params import GitCommitCreateParams as GitCommitCreateParams
from .config_schema_list_params import ConfigSchemaListParams as ConfigSchemaListParams
from .config_type_create_params import ConfigTypeCreateParams as ConfigTypeCreateParams
from .config_type_list_response import ConfigTypeListResponse as ConfigTypeListResponse
from .config_type_update_params import ConfigTypeUpdateParams as ConfigTypeUpdateParams
from .deployment_archive_params import DeploymentArchiveParams as DeploymentArchiveParams
from .deployment_retrieve_params import DeploymentRetrieveParams as DeploymentRetrieveParams
from .config_instance_list_params import ConfigInstanceListParams as ConfigInstanceListParams
from .config_schema_create_params import ConfigSchemaCreateParams as ConfigSchemaCreateParams
from .config_instance_create_params import ConfigInstanceCreateParams as ConfigInstanceCreateParams
from .config_instance_list_response import ConfigInstanceListResponse as ConfigInstanceListResponse
from .config_schema_retrieve_params import ConfigSchemaRetrieveParams as ConfigSchemaRetrieveParams
from .deployment_list_drifts_params import DeploymentListDriftsParams as DeploymentListDriftsParams
from .config_instance_download_params import ConfigInstanceDownloadParams as ConfigInstanceDownloadParams
from .config_instance_retrieve_params import ConfigInstanceRetrieveParams as ConfigInstanceRetrieveParams
from .device_issue_activation_token_params import DeviceIssueActivationTokenParams as DeviceIssueActivationTokenParams
from .device_issue_activation_token_response import (
    DeviceIssueActivationTokenResponse as DeviceIssueActivationTokenResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["DeviceIssueActivationTokenResponse"]


class DeviceIssueActivationTokenResponse(BaseModel):
    token: str
    """The token."""

    expires_at: datetime
    """The expiration date and time of the token."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["DeploymentDeployParams"]


class DeploymentDeployParams(TypedDict, total=False):
    expand: List[Literal["device", "release", "config_instances"]]
    """Fields to expand on the deployment resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["InstanceContentParam"]


class InstanceContentParam(TypedDict, total=False):
    data: Required[str]
    """The configuration values associated with the config instance."""

    format: Required[Literal["json"]]
